@@ -15,9 +15,23 @@ Usage (module):
     import jc.parsers.finger
     result = jc.parsers.finger.parse(finger_command_output)
 
-Compatibility:
+Schema:
 
-    'linux', 'darwin', 'cygwin', freebsd'
+    [
+      {
+        "login":                string,
+        "name":                 string,
+        "tty":                  string,
+        "idle":                 string,     # null if empty
+        "login_time":           string,
+        "details":              string,
+        "tty_writeable":        boolean,
+        "idle_minutes":         integer,
+        "idle_hours":           integer,
+        "idle_days":            integer,
+        "total_idle_minutes":   integer
+      }
+    ]
 
 Examples:
 
@@ -77,7 +91,8 @@ import jc.parsers.universal
 
 
 class info():
-    version = '1.0'
+    """Provides parser metadata (version, author, etc.)"""
+    version = '1.1'
     description = '`finger` command parser'
     author = 'Kelly Brazil'
     author_email = 'kellyjonbrazil@gmail.com'
@@ -91,7 +106,7 @@ class info():
 __version__ = info.version
 
 
-def process(proc_data):
+def _process(proc_data):
     """
     Final processing to conform to the schema.
 
@@ -101,23 +116,7 @@ def process(proc_data):
 
     Returns:
 
-        List of Dictionaries. Structured data with the following schema:
-
-        [
-          {
-            "login":                string,
-            "name":                 string,
-            "tty":                  string,
-            "idle":                 string,     # null if empty
-            "login_time":           string,
-            "details":              string,
-            "tty_writeable":        boolean,
-            "idle_minutes":         integer,
-            "idle_hours":           integer,
-            "idle_days":            integer,
-            "total_idle_minutes":   integer
-          }
-        ]
+        List of Dictionaries. Structured data to conform to the schema.
     """
     for entry in proc_data:
         if 'tty' in entry:
@@ -212,4 +211,4 @@ def parse(data, raw=False, quiet=False):
     if raw:
         return raw_output
     else:
-        return process(raw_output)
+        return _process(raw_output)
